@@ -1,17 +1,18 @@
 // Copyright (C) 2016 The Qt Company Ltd and/or its subsidiary(-ies).
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include "avfcameradebug_p.h"
 #include "qavfcamerabase_p.h"
-#include "avfcamerautility_p.h"
-#include <private/qcameradevice_p.h>
-#include "qavfhelpers_p.h"
-#include <private/qplatformmediaintegration_p.h>
+#include <QtMultimedia/private/qavfcameradebug_p.h>
+#include <QtMultimedia/private/qavfcamerautility_p.h>
+#include <QtMultimedia/private/qavfhelpers_p.h>
+#include <QtMultimedia/private/qcameradevice_p.h>
+#include <QtMultimedia/private/qplatformmediaintegration_p.h>
+
+#include <QtCore/qcoreapplication.h>
+#include <QtCore/qmetaobject.h>
+#include <QtCore/qpermissions.h>
 #include <QtCore/qset.h>
 #include <QtCore/qsystemdetection.h>
-#include <QMetaEnum>
-#include <QtCore/qcoreapplication.h>
-#include <QtCore/qpermissions.h>
 
 QT_USE_NAMESPACE
 
@@ -96,196 +97,8 @@ bool qt_convert_exposure_mode(AVCaptureDevice *captureDevice, QCamera::ExposureM
 
 #endif // defined(Q_OS_IOS)
 
-// Helper function to translate AVCaptureDevicePosition enum to QCameraDevice::Position enum.
-QCameraDevice::Position qt_AVCaptureDevicePosition_to_QCameraDevicePosition(AVCaptureDevicePosition input)
-{
-    switch (input) {
-        case AVCaptureDevicePositionFront:
-            return QCameraDevice::Position::FrontFace;
-        case AVCaptureDevicePositionBack:
-            return QCameraDevice::Position::BackFace;
-        case AVCaptureDevicePositionUnspecified:
-            return QCameraDevice::Position::UnspecifiedPosition;
-        default:
-            return QCameraDevice::Position::UnspecifiedPosition;
-    }
-}
-
 } // Unnamed namespace.
 
-
-
-QAVFVideoDevices::QAVFVideoDevices(QPlatformMediaIntegration *integration)
-    : QPlatformVideoDevices(integration)
-{
-    NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
-    m_deviceConnectedObserver = [notificationCenter addObserverForName:AVCaptureDeviceWasConnectedNotification
-                                                                object:nil
-                                                                queue:[NSOperationQueue mainQueue]
-                                                                usingBlock:^(NSNotification *) {
-                                                                        this->updateCameraDevices();
-                                                                }];
-
-    m_deviceDisconnectedObserver = [notificationCenter addObserverForName:AVCaptureDeviceWasDisconnectedNotification
-                                                                object:nil
-                                                                queue:[NSOperationQueue mainQueue]
-                                                                usingBlock:^(NSNotification *) {
-                                                                        this->updateCameraDevices();
-                                                                }];
-    updateCameraDevices();
-}
-
-QAVFVideoDevices::~QAVFVideoDevices()
-{
-    NSNotificationCenter* notificationCenter = [NSNotificationCenter defaultCenter];
-    [notificationCenter removeObserver:(id)m_deviceConnectedObserver];
-    [notificationCenter removeObserver:(id)m_deviceDisconnectedObserver];
-}
-
-QList<QCameraDevice> QAVFVideoDevices::videoDevices() const
-{
-    return m_cameraDevices;
-}
-
-void QAVFVideoDevices::updateCameraDevices()
-{
-#ifdef Q_OS_IOS
-    // Cameras can't change dynamically on iOS. Update only once.
-    if (!m_cameraDevices.isEmpty())
-        return;
-#endif
-
-    QList<QCameraDevice> cameras;
-
-    // List of all capture device types that we want to discover. Seems that this is the
-    // only way to discover all types. This filter is mandatory and has no "unspecified"
-    // option like AVCaptureDevicePosition(Unspecified) has. Order of the list is important
-    // because discovered devices will be in the same order and we want the first one found
-    // to be our default device.
-    NSArray *discoveryDevices = @[
-#ifdef Q_OS_IOS
-      AVCaptureDeviceTypeBuiltInTripleCamera,    // We always  prefer triple camera.
-      AVCaptureDeviceTypeBuiltInDualCamera,      // If triple is not available, we prefer
-                                                 // dual with wide + tele lens.
-      AVCaptureDeviceTypeBuiltInDualWideCamera,  // Dual with wide and ultrawide is still
-                                                 // better than single.
-#endif
-      AVCaptureDeviceTypeBuiltInWideAngleCamera, // This is the most common single camera type.
-                                                 // We prefer that over tele and ultra-wide.
-#ifdef Q_OS_IOS
-      AVCaptureDeviceTypeBuiltInTelephotoCamera, // Cannot imagine how, but if only tele and
-                                                 // ultrawide are available, we prefer tele.
-      AVCaptureDeviceTypeBuiltInUltraWideCamera,
-#endif
-    ];
-
-#if QT_DARWIN_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_14_0, __IPHONE_17_0, __TVOS_NA, __WATCHOS_NA)
-    if (@available(macOS 14, iOS 17, *)) {
-      discoveryDevices = [discoveryDevices arrayByAddingObjectsFromArray: @[
-        AVCaptureDeviceTypeExternal,
-        AVCaptureDeviceTypeContinuityCamera
-      ]];
-    } else
-#endif
-    {
-#ifdef Q_OS_MACOS
-    QT_WARNING_PUSH
-    QT_WARNING_DISABLE_DEPRECATED
-        discoveryDevices = [discoveryDevices arrayByAddingObjectsFromArray: @[
-            AVCaptureDeviceTypeExternalUnknown
-        ]];
-    QT_WARNING_POP
-#endif
-    }
-    // Create discovery session to discover all possible camera types of the system.
-    // Both "hard" and "soft" types.
-    AVCaptureDeviceDiscoverySession *discoverySession = [AVCaptureDeviceDiscoverySession
-                                                         discoverySessionWithDeviceTypes:discoveryDevices
-                                                         mediaType:AVMediaTypeVideo
-                                                         position:AVCaptureDevicePositionUnspecified];
-    NSArray<AVCaptureDevice *> *videoDevices = discoverySession.devices;
-
-    for (AVCaptureDevice *device in videoDevices) {
-        auto info = std::make_unique<QCameraDevicePrivate>();
-        if ([videoDevices[0].uniqueID isEqualToString:device.uniqueID])
-            info->isDefault = true;
-        info->id = QByteArray([[device uniqueID] UTF8String]);
-        info->description = QString::fromNSString([device localizedName]);
-        info->position = qt_AVCaptureDevicePosition_to_QCameraDevicePosition([device position]);
-
-        qCDebug(qLcCamera) << "Handling camera info" << info->description
-                           << (info->isDefault ? "(default)" : "");
-
-        QSet<QSize> photoResolutions;
-        QList<QCameraFormat> videoFormats;
-
-        for (AVCaptureDeviceFormat *format in device.formats) {
-            if (![format.mediaType isEqualToString:AVMediaTypeVideo])
-                continue;
-
-            auto dimensions = CMVideoFormatDescriptionGetDimensions(format.formatDescription);
-            QSize resolution(dimensions.width, dimensions.height);
-            photoResolutions.insert(resolution);
-
-            float maxFrameRate = 0;
-            float minFrameRate = 1.e6;
-
-            auto encoding = CMVideoFormatDescriptionGetCodecType(format.formatDescription);
-            auto pixelFormat = QAVFHelpers::fromCVPixelFormat(encoding);
-            auto colorRange = QAVFHelpers::colorRangeForCVPixelFormat(encoding);
-            // Ignore pixel formats we can't handle
-            if (pixelFormat == QVideoFrameFormat::Format_Invalid) {
-                qCDebug(qLcCamera) << "ignore camera CV format" << encoding
-                                   << "as no matching video format found";
-                continue;
-            }
-
-            for (AVFrameRateRange *frameRateRange in format.videoSupportedFrameRateRanges) {
-                if (frameRateRange.minFrameRate < minFrameRate)
-                    minFrameRate = frameRateRange.minFrameRate;
-                if (frameRateRange.maxFrameRate > maxFrameRate)
-                    maxFrameRate = frameRateRange.maxFrameRate;
-            }
-
-#ifdef Q_OS_IOS
-            // From Apple's docs (iOS):
-            // By default, AVCaptureStillImageOutput emits images with the same dimensions as
-            // its source AVCaptureDevice instance’s activeFormat.formatDescription. However,
-            // if you set this property to YES, the receiver emits still images at the capture
-            // device’s highResolutionStillImageDimensions value.
-            const QSize hrRes(qt_device_format_high_resolution(format));
-            if (!hrRes.isNull() && hrRes.isValid())
-                photoResolutions.insert(hrRes);
-#endif
-
-            qCDebug(qLcCamera) << "Add camera format. pixelFormat:" << pixelFormat
-                               << "colorRange:" << colorRange << "cvPixelFormat" << encoding
-                               << "resolution:" << resolution << "frameRate: [" << minFrameRate
-                               << maxFrameRate << "]";
-
-            auto *f = new QCameraFormatPrivate{ QSharedData(), pixelFormat,  resolution,
-                                                minFrameRate,  maxFrameRate, colorRange };
-            videoFormats << f->create();
-        }
-        if (videoFormats.isEmpty()) {
-            // skip broken cameras without valid formats
-            qCWarning(qLcCamera())
-                    << "Skip camera" << info->description << "without supported formats";
-            continue;
-        }
-        info->videoFormats = videoFormats;
-        info->photoResolutions = photoResolutions.values();
-
-        cameras.append(info.release()->create());
-    }
-
-    if (cameras != m_cameraDevices) {
-        m_cameraDevices = cameras;
-        emit videoInputsChanged();
-    }
-}
-
-
 QAVFCameraBase::QAVFCameraBase(QCamera *camera)
    : QPlatformCamera(camera)
 {
@@ -332,13 +145,22 @@ void QAVFCameraBase::setCamera(const QCameraDevice &camera)
     updateCameraConfiguration();
 }
 
-// Currently not used by any backend.
-bool QAVFCameraBase::setCameraFormat(const QCameraFormat &format)
+bool QAVFCameraBase::setCameraFormat(const QCameraFormat &newFormat)
 {
-    if (!format.isNull() && !m_cameraDevice.videoFormats().contains(format))
+    if (!newFormat.isNull() && !m_cameraDevice.videoFormats().contains(newFormat))
         return false;
 
-    m_cameraFormat = format.isNull() ? findBestCameraFormat(m_cameraDevice) : format;
+    const QCameraFormat resolvedFormat = newFormat.isNull() ? findBestCameraFormat(m_cameraDevice) : newFormat;
+    // If we still couldn't find a suitable default format (such as when none are available)
+    // we don't accept the value.
+    if (resolvedFormat.isNull())
+        return false;
+
+    const bool applySuccess = tryApplyCameraFormat(resolvedFormat);
+    if (!applySuccess)
+        return false;
+
+    m_cameraFormat = resolvedFormat;
 
     return true;
 }
@@ -378,10 +200,10 @@ void QAVFCameraBase::setFocusMode(QCamera::FocusMode mode)
 void QAVFCameraBase::forceSetFocusMode(QCamera::FocusMode mode)
 {
     if (!isFocusModeSupported(mode)) {
-        qCDebug(qLcCamera) <<
-            Q_FUNC_INFO <<
-            QString("attempted to set focus-mode '%1' on camera where it is unsupported.")
-                .arg(QMetaEnum::fromType<QCamera::FocusMode>().valueToKey(mode));
+        qCDebug(qLcCamera)
+                << Q_FUNC_INFO
+                << QString(u"attempted to set focus-mode '%1' on camera where it is unsupported.")
+                           .arg(QMetaEnum::fromType<QCamera::FocusMode>().valueToKey(mode));
         return;
     }
 
@@ -418,7 +240,7 @@ void QAVFCameraBase::forceSetFocusMode(QCamera::FocusMode mode)
         captureDevice.focusMode = AVCaptureFocusModeContinuousAutoFocus;
     }
 
-    Q_EMIT focusModeChanged(mode);
+    focusModeChanged(mode);
 }
 
 bool QAVFCameraBase::isFocusModeSupported(QCamera::FocusMode mode) const
