@@ -7,7 +7,6 @@
 #include "evrhelpers_p.h"
 #include <private/qwindowsmultimediautils_p.h>
 #include <private/qplatformvideosink_p.h>
-#include <private/qwindowsmfdefs_p.h>
 
 #include <rhi/qrhi.h>
 
@@ -26,13 +25,15 @@
 
 QT_BEGIN_NAMESPACE
 
-static Q_LOGGING_CATEGORY(qLcEvrCustomPresenter, "qt.multimedia.evrcustompresenter")
+Q_STATIC_LOGGING_CATEGORY(qLcEvrCustomPresenter, "qt.multimedia.evrcustompresenter");
 
 const static MFRatio g_DefaultFrameRate = { 30, 1 };
 static const DWORD SCHEDULER_TIMEOUT = 5000;
 static const MFTIME ONE_SECOND = 10000000;
 static const LONG   ONE_MSEC = 1000;
 
+#define QMM_PRESENTATION_CURRENT_POSITION 0x7fffffffffffffff
+
 // Function declarations.
 static HRESULT setMixerSourceRect(IMFTransform *mixer, const MFVideoNormalizedRect& nrcSource);
 static QVideoFrameFormat::PixelFormat pixelFormatFromMediaType(IMFMediaType *type);
@@ -404,7 +405,6 @@ ComPtr<IMFSample> SamplePool::takeSample()
 {
     QMutexLocker locker(&m_mutex);
 
-    Q_ASSERT(m_initialized);
     if (!m_initialized) {
         qCWarning(qLcEvrCustomPresenter) << "SamplePool is not initialized yet";
         return nullptr;
@@ -626,12 +626,10 @@ HRESULT EVRCustomPresenter::InitServicePointers(IMFTopologyServiceLookup *lookup
 HRESULT EVRCustomPresenter::ReleaseServicePointers()
 {
     // Enter the shut-down state.
-    m_mutex.lock();
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     m_renderState = RenderShutdown;
 
-    m_mutex.unlock();
-
     // Flush any samples that were scheduled.
     flush();
 
